@@ -1,25 +1,48 @@
 #!/bin/sh
 
-Script_Ver="1.04.2013"
+Script_Ver="1.04.5701"
 
 CheckCpuType(){
 	ARCH=`uname -m`
 	echo "(I) Platform application binary interface = ${ARCH}"
 	if [ $ARCH = "x86_64" ] ; then
 		cpuArch="64"
-	elif [ $ARCH = "i386" -o $ARCH = "i686" ] ; then
+	elif [ $ARCH = "i386" -o $ARCH = "i586" -o $ARCH = "i686" ] ; then
 		cpuArch="32"
 	else
+		ReadELF="readelf -A /bin/sh"
+		tmpfile="tmp.info"
+		${ReadELF}>${tmpfile} 2>&1
+		CheckCondition1="Tag_ABI_VFP_args: VFP registers"
+		CheckCondition2="Tag_GNU_MIPS_ABI_FP"
+		grep -q "${CheckCondition1}" ${tmpfile}
+		if [ $? = 0 ];then
+			echo " "
+			echo "Your machine is detected may be as an ARM hard floating platform"
+			echo "it is recommended to select [2] ARMhf to install "	
+			echo " "
+		fi
+		grep -q "${CheckCondition2}" ${tmpfile}
+		if [ $? = 0 ];then
+			echo " "
+			echo "Your machine is detected may be as a MIPS platform"
+			echo "it is recommended to select [3] MIPS to install "	
+			echo " "
+		fi
+		rm ${tmpfile}
+
 		echo "Which platform arch do you use?"
-		echo "[1] ARM  [2] MIPS"
+		echo "[1] ARM  [2] ARMhf  [3] MIPS"
 		while : ; do
 			read arch
 			case $arch in
 				1)	cpuArch="ARM"
 					break;;
-				2)	cpuArch="MIPS"
+				2)	cpuArch="ARMhf"
 					break;;
-				*)	echo "(I) Please choose [1] or [2]"
+				3)	cpuArch="MIPS"
+					break;;
+				*)	echo "(I) Please choose [1], [2] or [3]"
 					echo -n "(A) ";;
 			esac
 		done
@@ -29,6 +52,7 @@ CheckCpuType(){
 InitMember(){
 	# file name
 	drivername="eGTouch"
+	eGTouchSH="eGTouch.sh"
 	xorgfile="52-egalax-virtual.conf"
 	calibTool="eCalib"
 	Distri="GENERAL"
@@ -47,6 +71,10 @@ InitMember(){
 	installpath="/usr/local"
 	etcpath="/etc"
 	usrbinpath="/usr/bin"
+	initpath="/etc/init.d"
+	rcSpath="/etc/rcS.d"
+	rc5path="/etc/rc5.d"
+	lightdmpath="/etc/lightdm/lightdm.conf"
 	rclocalfile="rc.local"
 	rclocalpath="/etc/rc.local"				# Ubuntu and Fedora, General OS
 	rclocalpath1="/etc/init.d/boot.local"	# OpenSuSe 11.04.
@@ -55,6 +83,7 @@ InitMember(){
 	etcModuleFile1="/etc/modules"				# Ubuntu modules bootup file
 	etcModuleFile2="/etc/modules.conf"			# RedHat modules bootup file
 	etcModuleFile3="/etc/conf.modules"			# SuSe modules bootup file
+	etcModuleFile4="/etc/sysconfig/kernel"      # Suse 11.4 module loaded on bootup file
 	trash="/dev/null"
 	xorgpath="/usr/share/X11/xorg.conf.d"	# Ubuntu and Fedora
 	xorgpath1="/etc/X11/xorg.conf.d"		# OpenSuSe 11.04
@@ -67,8 +96,6 @@ InitMember(){
 
 	#For PS2 interface
 	seriofile="serio_raw.sh"
-	initpath="/etc/init.d"
-	rcSpath="/etc/rcS.d"
 	rcSfile="S01serio_raw.sh"
 
 	Xexist="true"
@@ -149,7 +176,7 @@ InstallDriverRoutine() {
 		fi
 		
 		if [ -e ${sysDrvFldr}/${calibTool} ]; then
-			chmod a+x ${sysDrvFldr}/${calibTool}
+			chmod 4755 ${sysDrvFldr}/${calibTool}
 			ln -sf ${sysDrvFldr}/${calibTool} ${usrbinpath}
 			echo "(I) Create ${calibTool} tool shortcut in ${usrbinpath}."
 		fi
@@ -157,6 +184,8 @@ InstallDriverRoutine() {
         echo "(W) There is no directory ${usrbinpath}. We can NOT link shortcut under ${usrbinpath}."
 		echo "(W) Please link it manually."
     fi
+
+	cp setup.sh ${sysDrvFldr}
 }
 
 CheckUSBType() {
@@ -179,16 +208,46 @@ CheckUSBType() {
     rm -f ${usbfile}
 }
 
+CheckUSBPIDnKernel() {
+    Kversion=`uname -r`
+    Kversion=${Kversion##* }	#3.8.0-23-generic-pae
+    Kmajor=${Kversion%%.*}	#3
+    Ktmp=${Kversion#*.}
+    Kminor=${Ktmp%%.*}		#8
+
+    if [ $Kmajor -ge 3 -a $Kminor -ge 8 ] && [ $Kmajor -le 3 -a $Kminor -le 12 ]  ;then
+	usbfile="usb.info"
+	checkusbpid="lsusb -d 0eef:"
+	${checkusbpid} > ${usbfile} 2> ${trash}
+	grep -q "0001" ${usbfile}
+	if [ $? -eq 0 ]; then
+	    echo ""
+	    echo "(I) Found a PID:0001 touch controller in kernel 3.8 upwards."
+            dmesg | grep "hid-multitouch 0003:0EEF:0001" > ${trash}
+	    if [ $? -ne 0 ]; then	        
+	        RemindKernelHIDPatch
+	    fi
+	fi
+	rm -f ${usbfile}
+    fi
+
+}
+
 GenerateSerioRaw(){
 	checkcmd="which update-rc.d"
 	cmdinfofile="cmd.info"
 	${checkcmd} > ${cmdinfofile} 2>&1
 	grep -q "no update-rc.d in" ${cmdinfofile}
-	if [ $? -eq 0 ]; then
-		filelines=`cat ${rclocalpath} | wc -l`
-		sed -i ''${filelines}'i\# added by eGalaxTouch installer\necho -n "serio_raw" > /sys/bus/serio/devices/'${serioport}'/drvctl' ${rclocalpath}
-		echo "(I) Generate serio script in $rclocalpath"
-	else
+	if [ $? -eq 0 ]; then #Put it into rclocal. Generally, we'll go into this part.
+		grep -q "added by eGalaxTouch installer" $rclocalModulesPath
+		if [ $? = 0 ]; then
+			echo "(I) echo seriow_raw was already written in ${rclocalModulesPath}"
+		else
+			filelines=`cat ${rclocalModulesPath} | wc -l`
+			sed -i ''${filelines}'i\# added by eGalaxTouch installer\necho -n "serio_raw" > /sys/bus/serio/devices/'${1}'/drvctl' ${rclocalModulesPath}
+			echo "(I) Generate serio script in $rclocalModulesPath"
+		fi
+	else # This part seems like not do the verification. I'm not sure that it could work properly.
 		cp -f Rule/${seriofile} ${initpath}
 		chmod 755 $initpath/$seriofile
 		sed -i '13a echo -n "serio_raw" > /sys/bus/serio/devices/'${1}'/drvctl' ${initpath}/${seriofile}
@@ -198,7 +257,33 @@ GenerateSerioRaw(){
 	rm -f ${cmdinfofile}
 }
 
+CheckSerioRawModule(){
+	lsmod | grep serio_raw > ${trash}
+	if [ $? = 0 ];then
+		echo "(I) Module serio_raw.ko is detected under lsmod"
+	else
+		echo "(W) Module \"serio_raw.ko\" doesn't exist in kernel module."
+		modprobe serio_raw
+		if [ $? = 0 ];then
+			echo "(I) Module serio_raw could be successfully loaded by modprobe."
+			AttachModuleAtBoot "serio_raw"
+			if [ $? = 0 ];then
+				echo "(I) Attach insmod \"serio_raw\" at boot successfully."
+			else
+				echo "(E) Module \"serio_raw\" attached at boot failed."
+				echo "(E) Please mannually let the module \"serio_raw\" attached at boot. Or touch would not be workable."
+			fi
+		else
+			echo "(E) There is no module serio_raw.ko"
+			echo "(E) Please make sure it is built-in the kernel, or touch would not be workable."
+		fi
+	fi
+}
+
+
+
 SetPS2Config(){
+	CheckSerioRawModule
 	echo "(I) Configure PS/2 aux driver."
 	SerioDevPath="/sys/bus/serio/devices"
 	tmp_file="tmp.info"
@@ -213,11 +298,19 @@ SetPS2Config(){
 				grep -q "psmouse" ${tmp_file}
 				if [ $? -eq 0 ];then
 					serioport="serio${i}"
+					# If this serioX got PS mouse collection, at the file driver would link to ../bus/serio/drivers/psmouse 
 					echo "(I) Found PS2 mouse driver at serio${i}."
 					GenerateSerioRaw ${serioport}
 					rm -f ${tmp_file}
 					return
 				fi
+				
+				grep -q "serio_raw" ${tmp_file}
+				if [ $? = 0 ]; then
+					echo "(I) Found /"serio_raw/" located under ${SerioDevPath}/serio${i}/"
+					rm -f ${tmp_file}
+					return
+				fi
 			fi
 		done
 		echo "(E) No PS2 mouse driver found under ${SerioDevPath}/serioX"
@@ -279,22 +372,59 @@ AttachDrvExecAtRclocal() {
 
 DetachDrvExecFromRclocal() { # Remove the auto execution stings in rc.local
 	if [ -w ${rclocalModulesPath} ];then
-		if [ ${Distri} = "FC16" -o ${Distri} = "FC17" ];then
-	        rm -rf ${rclocalModulesPath}
-	    else
 			sed -i '/### Beginning: Launch eGTouchD daemon while setup boot-up ###/,/### End: Launch eGTouchD daemon while setup boot-up ###/d' ${rclocalModulesPath}
 			echo "(I) Detach eGTouchD daemon execution from ${rclocalModulesPath}."
-		fi
 	else
 		echo "(E) No ${rclocalModulesPath} file found."
 	fi
 }
 
+CheckRCExist(){
+	cp -f "Rule/${eGTouchSH}" ${initpath}
+	chmod a+x ${initpath}/${eGTouchSH}
+	if [ -e ${rcSpath} ];then # /etc/rcS.d exist
+		echo "(I) ${rcSpath} path found."
+		ln -s "${initpath}/${eGTouchSH}" "${rcSpath}/S99eGTouch" 
+	elif [ -e ${rc5path} ];then
+		echo "(I) ${rc5path} path found."
+		ln -s ${initpath}/${eGTouchSH} ${rc5path}/S99eGTouch 
+	fi
+}
+
+RemoveRCSetting(){
+	rm "${initpath}/${eGTouchSH}"
+	if [ -e ${rcSpath} ];then # /etc/rcS.d exist
+		echo "(I) ${rcSpath} path found."
+		rm "${rcSpath}/S99eGTouch"
+	elif [ -e ${rc5path} ];then
+		echo "(I) ${rc5path} path found."
+		rm "${rc5path}/S99eGTouch"
+	fi
+}
+
+CheckLightDMExist(){
+	if [ -e ${lightdmpath} ];then # lightdm is exist
+		echo "(I) ${lightdmpath} file found."
+		sed -i '/\[SeatDefaults\]/asession-setup-script=\/usr\/bin\/eGTouchD' ${lightdmpath}
+	fi
+}
+
+RemoveLightDMSetting(){
+	if [ -e ${lightdmpath} ];then # lightdm is exist
+		echo "(I) ${lightdmpath} file found."
+		sed -i '/eGTouchD/d' ${lightdmpath}
+	fi
+}
+
+
 CheckRClocalExist(){
 	if [ ! -e ${rclocalModulesPath} ];then # rc.local is not exist
-		if [ ${Distri} = "FC16" -o ${Distri} = "FC17" ];then
+		if [ ${Distri} = "FC16" -o ${Distri} = "FC17" -o ${Distri} = "FC18" -o ${Distri} = "FC19" -o ${Distri} = "FC20" -o ${Distri} = "FC21" -o ${Distri} = "FC22" ];then
 			echo "(I) Copy ${rclocalfile} file to ${rclocalModulesPath}."
 			cp -f "Rule/${rclocalfile}" ${rclocalModulesPath}
+			chmod a+x ${rclocalModulesPath}
+			echo "(I) Starting rc-local.service"
+			systemctl restart rc-local.service
 		else
 			echo "(E) No ${rclocalModulesPath} file found."
 			RemoveDriver
@@ -309,8 +439,9 @@ CheckRClocalExist(){
 AllotRClocalPath(){ # This function would get target distri's rclocal path. And check whether it exist.		
 	if [ ${Distri} = "SUSE" ];then
 		rclocalModulesPath=${rclocalpath1}
-	elif [ ${Distri} = "FC14" -o ${Distri} = "FC16" -o ${Distri} = "FC17" -o ${Distri} = "Redhat6" ];then
+	elif [ ${Distri} = "FC14" -o ${Distri} = "FC16" -o ${Distri} = "FC17" -o ${Distri} = "FC18" -o ${Distri} = "FC19" -o ${Distri} = "FC20" -o ${Distri} = "FC21" -o ${Distri} = "FC22" -o ${Distri} = "Redhat6" -o ${Distri} = "CentOS6.3" -o ${Distri} = "CentOS6.4" -o ${Distri} = "CentOS6.2" -o ${Distri} = "Slackware" -o ${Distri} = "CentOS6.5" -o ${Distri} = "CentOS7.0" -o ${Distri} = "CentOS6.6" ];then
 		rclocalModulesPath=${rclocalpath2}
+		chmod 755 /etc/rc.d/rc.local
 	else
 		rclocalModulesPath=${rclocalpath}
 	fi
@@ -331,13 +462,19 @@ Add2Blacklist() {
         grep -q $1 ${blacklistpath}
         if [ $? -eq 1 ]; then
             filelines=`cat ${blacklistpath} | wc -l`
-            echo "(I) Add kernel module $1 into ${blacklistpath}."
-            sed -i ''${filelines}'a\### Beginning: blacklist usbtouchscreen ###\
+            if [ ${filelines} > 1 ];then
+                echo "(I) Add kernel module $1 into ${blacklistpath}."
+                sed -i ''${filelines}'a\### Beginning: blacklist usbtouchscreen ###\
 blacklist usbtouchscreen\
 ### End: blacklist usbtouchscreen ###' ${blacklistpath}
+            else
+		        echo -e "\n### Beginning: blacklist usbtouchscreen ###\nblacklist usbtouchscreen\n### End: blacklist usbtouchscreen ###" > ${blacklistpath}
+            fi
         else
             echo "(I) The kernel module $1 has been added in ${blacklistpath}."
         fi
+    else
+	echo -e "\n### Beginning: blacklist usbtouchscreen ###\nblacklist usbtouchscreen\n### End: blacklist usbtouchscreen ###" > ${blacklistpath}
     fi
 }
 
@@ -453,6 +590,11 @@ RemoveDriver() {
     RemoveShortcut
 	RemoveSerioScript
     DetachDrvExecFromRclocal
+	
+	if [ ${Distri} = "FC16" -o ${Distri} = "FC17" -o ${Distri} = "FC18" -o ${Distri} = "FC19" -o ${Distri} = "FC20" -o ${Distri} = "FC21" -o ${Distri} = "FC22" ];then
+        rm -rf ${rclocalModulesPath}
+	fi
+
     RestoreBlacklist
     DetachModuleAtBoot "uinput"
 	if [ $Xexist = "true" ];then
@@ -541,7 +683,7 @@ CheckUinput(){
 }
 
 AttachModuleAtBoot(){
-	echo "(I) Attach $1 at boot."
+	echo "(I) Attach module $1 loaded at boot."
     if [ -w ${etcModuleFile1} ]; then
         grep -q "### Beginning: Load ${1}.ko modules ###" ${etcModuleFile1}
         if [ $? -eq 1 ]; then 
@@ -569,13 +711,22 @@ AttachModuleAtBoot(){
 ### End: Load '${1}'.ko modules###' ${etcModuleFile3}
 			echo "(I) Add ${1} module into ${etcModuleFile3} file."
 		fi
+	elif [ -w ${etcModuleFile4} ]; then
+		grep -q "### Beginning: Load ${1}.ko modules ###" ${etcModuleFile4}
+		if [ $? -eq 1 ]; then
+			filelines=`cat ${etcModuleFile4} | wc -l`
+			sed -i ''${filelines}'a\### Beginning: Load '${1}'.ko modules ###\
+MODULES_LOADED_ON_BOOT="'${1}'"\
+### End: Load '${1}'.ko modules###' ${etcModuleFile4}
+			echo "(I) Add ${1} module into ${etcModuleFile4} file."
+		fi
 	elif [ -w ${rclocalModulesPath} ]; then
 		grep -q "### Beginning: Load ${1}.ko modules ###" ${rclocalModulesPath}
 		if [ $? -eq 1 ]; then
 			filelines=`cat ${rclocalModulesPath} | wc -l`
-			echo "(I) Add '${1}' module into ${rclocalModulesPath} file."
+			echo "(I) Add modprobe '${1}' module into ${rclocalModulesPath} file."
 			sed -i ''${filelines}'a\### Beginning: Load '${1}'.ko modules ###\
-'${1}'\
+modprobe '${1}'\
 ### End: Load '${1}'.ko modules###' ${rclocalModulesPath}
 		fi
 	else
@@ -625,13 +776,27 @@ GetDistribution(){
 	${Xcommand}>${tmpfile} 2>&1
 	
 	CheckCondition1="SUSE"
-	CheckCondition2="Build ID: xorg-x11-server 1.9.0-15.fc14 "
-	CheckCondition3="Build ID: xorg-x11-server 1.7.7-26.el6"
-	CheckCondition4="Build ID: xorg-x11-server 1.11.1-1.fc16"
-	CheckCondition5="Build ID: xorg-x11-server 1.12.0-2.fc17"
-	CheckCondition6="xorg-server 2:1.7.6-2ubuntu7"
-	CheckCondition7="Build ID: xorg-x11-server 1.8.0-12.fc13"
-	
+	CheckCondition2="Build ID: xorg-x11-server 1.9.0-15.fc14 "		#FC14
+	CheckCondition3="Build ID: xorg-x11-server 1.7.7-26.el6"		#RHEL 6.1
+	CheckCondition4="Build ID: xorg-x11-server 1.11.1-1.fc16"		#FC16
+	CheckCondition5="Build ID: xorg-x11-server 1.12.0-2.fc17"		#FC17
+	CheckCondition6="xorg-server 2:1.7.6-2ubuntu7"				#Ubuntu 10.04
+	CheckCondition7="Build ID: xorg-x11-server 1.8.0-12.fc13"		#FC13
+	CheckCondition8="Build ID: xorg-x11-server 1.10.6-1.el6.centos"		#CentOS 6.3
+	CheckCondition9="Build ID: xorg-x11-server 1.13.0-11.el6.centos"	#CentOS 6.4
+	CheckCondition10="Build ID: xorg-x11-server 1.10.4-6.el6"		#CentOS 6.2
+	CheckCondition11="Build ID: xorg-x11-server 1.13.0-11.el6"		#RHEL 6.4
+	CheckCondition12="Build ID: xorg-x11-server 1.13.0-11.fc18"		#FC18
+	CheckCondition13="Build ID: xorg-x11-server 1.14.1-4.fc19"		#FC19
+	CheckCondition14="Build ID: xorg-x11-server 1.10.6-1.el6"		#RHEL 6.3
+	CheckCondition15="Slackware Linux Project"				#Slackware
+	CheckCondition16="Build ID: xorg-x11-server 1.14.4-5.fc20"		#FC20
+	CheckCondition17="Build ID: xorg-x11-server 1.13.0-23"			#CentOS 6.5
+	CheckCondition18="Build ID: xorg-x11-server 1.15.0-7.el7"		#CentOS 7.0
+	CheckCondition19="Build ID: xorg-x11-server 1.15.0-22.el6.centos"	#CentOS 6.6
+	CheckCondition20="Build ID: xorg-x11-server 1.16.1-1.fc21"		#FC21
+	CheckCondition21="Build ID: xorg-x11-server 1.17.1-11.fc22"		#FC22
+
 	grep -q "${CheckCondition1}" ${tmpfile}
 	if [ $? = 0 ];then
 		Distri="SUSE"
@@ -666,11 +831,95 @@ GetDistribution(){
 	if [ $? = 0 ];then
 		Distri="FC14"
 	fi
+
+	grep -q "${CheckCondition8}" ${tmpfile}
+	if [ $? = 0 ];then
+		Distri="CentOS6.3"
+	fi
+
+	grep -q "${CheckCondition9}" ${tmpfile}
+	if [ $? = 0 ];then
+		Distri="CentOS6.4"
+	fi
 	
+	grep -q "${CheckCondition10}" ${tmpfile}
+	if [ $? = 0 ];then
+		Distri="CentOS6.2"
+	fi
+	
+	grep -q "${CheckCondition11}" ${tmpfile}
+	if [ $? = 0 ];then
+		Distri="Redhat6"
+	fi
+
+	grep -q "${CheckCondition12}" ${tmpfile}
+	if [ $? = 0 ];then
+		Distri="FC18"
+	fi
+
+	grep -q "${CheckCondition13}" ${tmpfile}
+	if [ $? = 0 ];then
+		Distri="FC19"
+	fi
+
+	grep -q "${CheckCondition14}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="RedHat6"
+        fi
+
+        grep -q "${CheckCondition15}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="Slackware"
+        fi
+
+	grep -q "${CheckCondition16}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="FC20"
+        fi
+
+	grep -q "${CheckCondition17}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="CentOS6.5"
+        fi
+
+	grep -q "${CheckCondition18}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="CentOS7.0"
+        fi
+
+	grep -q "${CheckCondition19}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="CentOS6.6"
+        fi
+
+	grep -q "${CheckCondition20}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="FC21"
+        fi
+
+	grep -q "${CheckCondition21}" ${tmpfile}
+        if [ $? = 0 ];then
+                Distri="FC22"
+        fi
+		
 	rm ${tmpfile}
 	return 0
 }
 
+RemindKernelHIDPatch(){
+	echo ""
+	echo "(W) No hid-multitouch module detected"
+	echo "(W) Please follow the Programming Guide to patch hid-core source code in kernel."
+	echo ""
+	echo " [Y] Yes, I've patched kernel already.  [N] No, I haven't patched."
+	read ans
+	if [ ${ans} != "Y" -a ${ans} != "y" ];then
+		echo "(I) Please patch kernel before installing driver. Thanks."
+		echo ""
+		exit 1
+	fi
+}
+
 RemindKernelPatch(){
 	echo ""
 	echo "(W) You need to do kernel patch first."
@@ -695,7 +944,7 @@ CheckXversion(){
 	Xversion=`grep "${Xorg}" ${tmpfile}` # ex: Xversion = X.Org X Server 1.4.0.90.3
 	rm ${tmpfile}
 	
-	Xversion=${Xversion##* } # 1.4.0.90.3
+	Xversion=${Xversion##${Xorg} } # 1.4.0.90.3
 	Xmajor=${Xversion%%.*} # 1
 	Xtmp=${Xversion#*.} # 4.0.90.3
 	Xminor=${Xtmp%%.*}  # 4
@@ -704,7 +953,7 @@ CheckXversion(){
 	echo "(I) X.Org X server ${Xmajor}.${Xminor}.${Xrelease}" # 1.4.0
 	
 	if [ $Xmajor -ge 1 -a $Xminor -ge 8 ] || [ $Xmajor -ge 1 -a $Xminor -ge 7 -a $Xrelease -ge 6 ];then
-		echo "(I) X verion is 1.7.6 upwards"
+		echo "(I) X version is 1.7.6 upwards"
 	else # kernel version is below 1.8.7
 		echo "(W) X version is below 1.7.6."
 		RemindKernelPatch
@@ -807,10 +1056,13 @@ AllotRClocalPath
 if [ $# = 0 ]; then
 	PatentConfirm
 	AskInterface
-    pause
-	CheckRClocalExist
+	pause
+	CheckRCExist
+	#CheckRClocalExist
+	#CheckLightDMExist
 	if [ $interface = "usb" ];then
 		CheckUSBType
+		CheckUSBPIDnKernel
 		if [ $Xexist = "true" ];then
 			CheckXversion
 		else
@@ -821,8 +1073,8 @@ if [ $# = 0 ]; then
 	fi
 
 	CheckUinput
-    InstallDriverRoutine
-    ModifyRClocal
+	InstallDriverRoutine
+#	ModifyRClocal
 	AskDevNums
 	if [ $Xexist = "true" ];then
 		AttachUdevRule # Append udev rule onto X.org
@@ -830,25 +1082,27 @@ if [ $# = 0 ]; then
 	else
 		AskTslib
 	fi
-    echo ""
+	echo ""
 	echo "(I) Driver installation completed. Setup version $Script_Ver."
-    echo "(I) Please reboot the system."
+	echo "(I) Please reboot the system."
 	
 elif [ $# = 1 ]; then
-    if [ $1 = "uninstall" ]; then
-        echo "(I) Begin to remove ${drivername} driver."
-        RemoveDriver
+	if [ $1 = "uninstall" ]; then
+        	echo "(I) Begin to remove ${drivername} driver."
+		RemoveDriver
 		RemoveUtilityShortCut # Add/Remove Utility ShortCut
-        echo ""
-        echo "(I) The ${drivername} driver has been removed successfully."
-        echo "(I) Please reboot the system."
-    else
-        echo "(I) Usage:"
+		RemoveLightDMSetting
+		RemoveRCSetting
+        	echo ""
+        	echo "(I) The ${drivername} driver has been removed successfully."
+        	echo "(I) Please reboot the system."
+	else
+		echo "(I) Usage:"
 		echo "(I) sh $0:           install driver package"
 		echo "(I) sh $0 uninstall: uninstall driver package"
-    fi
+	fi
 else
-    echo "(I) Usage:"
+	echo "(I) Usage:"
 	echo "(I) sh $0:           install driver package"
 	echo "(I) sh $0 uninstall: uninstall driver package"
 fi
